//----------------------------------------------
//Compiler    : Hi-Tech  V7.86 PL3
//File        : Milli Watt Meter
//Edition     : Jan 2002 OZ2CPU (Ver 1.03)
//CPU         : PIC16F876 @ 4Mhz
//HardwareInfo: AD8307 -60 to +30dBm 
//Powertip PC2002-B  LCD 20 * 2 lines
//RB7=Select  RB5=A Encoder RB0=B Encoder  RB6=MENU
//AN1=DBM input  AN0=DC Voltmeter input  AN3=2.5Vref input
//More info and latest updated version at www.webx.dk
//Copyright note: not for sale or any commercial use,
//ONLY ELEKTOR MAGAZINE and OZ2CPU may sell programmed PIC's
//ONLY ELEKTOR MAGAZINE have special rights to put this source code set online
//ELEKTOR and OZ2CPU gives no support and no warranties on this source code,
//use it for study and learn how to programm a PIC using C
//---------------------------------------------------------------
#include <xc.h>
#include<pic16f876.h>
//#include <pic.h>     // to compiler, enter PIC mode.
//#include "def.h"     // Define vars, constants and macros
//#include "txt.h"     // some text for Help and About screens
//#include "konv.c"    // konvert from integer to ascii
//#include "serial.c"  // Software serial transmit routines
//#include "delays.c"  // delay routines
//#include "lcd.c"     // routiner to controll most standard LCD modules
//#include "adc.c"     // all about the Analog to Digital converter
//#include "subs.c"    // Here is alot varius routines
//#include "info.c"    // help and about text screens
//#include "measure.c" // all measure and display routines
//#include "init.c"    // hardware and vars
#pragma config "CP = OFF", "WRT = OFF", "DEBUG = OFF"
#pragma config "CPD = ON", "WDTE = ON", "FOSC = EXTRC"
#pragma config "BOREN = ON"
#pragma config "IDLOC1=1", "IDLOC2=0", "IDLOC3=3" //version 103
//__IDLOC(103);  // Identify Locations 

// All variables, bits, and hardware ports are defined here
// For the Digital Wattmeter PCB version 1.3 OZ2CPU
// def.h
 unsigned char   LCDPORT @ 0x75;
 unsigned char   user1;
 unsigned char   user2;
 unsigned char   user3;
 unsigned char   user4;
 unsigned char   user5;
 unsigned char   wpref;
 unsigned char   wpointer;
 unsigned char   w2;
 unsigned char   w1;
 unsigned char   wpoint;
 unsigned char   vpref;     // prefix u m v kv
 unsigned char   vpointer;  // pointer to array
 unsigned char   v2;       // decimal part
 unsigned char   v1;       // msb part
 unsigned char   vpoint;  // where to place decimal point                        
 unsigned int    peak;
 unsigned int    rfraw;
 unsigned int    rfrawB;
 unsigned int    rfraw2;  // compensated with calibration value
 unsigned int    rfRawAtt; // compensated and with att values
 unsigned int    rfdbm;
 unsigned char   dbswr;  // db value used for the return loss to SWR calculation
 unsigned int    voltraw;
 unsigned int    voltmax;
 unsigned int    voltmin;
 unsigned int    konv;
 unsigned int    LFcal;
 unsigned int    HFcal;
 unsigned int    VHFcal;
 unsigned int    UHFcal;
 unsigned int    SHFcal;
 unsigned int    ActualCal; // selected CAL value in this var
 unsigned int    RelativeCal;  // the zero value when entering Relative mode
 unsigned char   message1;  // to center of display
 unsigned char   message2;
 unsigned char   message3;
 unsigned char   messagenumber;
 unsigned char   messagetimer;
 unsigned char   messagetype;
 unsigned char   bandmode;  // in what freq band selected
 unsigned char   attmode; 
 unsigned char   menutype;
 unsigned char   displaydelay;  // sets the screen delay in mS
 unsigned char   peakholddelay; // for the SSB power screen
 unsigned char   metertype;     // RF, SSB, DC, RTN, ABOUT osv...


///////////////////////// bitfield :Flags  ///////////////////////////////
 
  bit	ENA    @ (unsigned)&PORTC*8+3 ;  // LCD controll BIT in PORTC
  bit	RW     @ (unsigned)&PORTC*8+2 ;
  bit	RS     @ (unsigned)&PORTC*8+0 ;

  bit	LC0       @ (unsigned)&LCDPORT*8+0 ;
  bit	LC1       @ (unsigned)&LCDPORT*8+1 ;
  bit	LC2       @ (unsigned)&LCDPORT*8+2 ;
  bit	LC3       @ (unsigned)&LCDPORT*8+3 ;
  bit	LC4       @ (unsigned)&LCDPORT*8+4 ;
  bit	LC5       @ (unsigned)&LCDPORT*8+5 ;
  bit	LC6       @ (unsigned)&LCDPORT*8+6 ;
  bit	LC7       @ (unsigned)&LCDPORT*8+7 ;  // defined in variable

  bit	LCD0      @ (unsigned)&PORTC*8+1 ; // display actual bits defined
  bit	LCD1      @ (unsigned)&PORTB*8+4 ;
  bit	LCD2      @ (unsigned)&PORTB*8+3 ;
  bit	LCD3      @ (unsigned)&PORTB*8+2 ;
  bit	LCD4      @ (unsigned)&PORTB*8+1 ;
  bit	LCD5      @ (unsigned)&PORTC*8+4 ;
  bit	LCD6      @ (unsigned)&PORTC*8+5 ;
  bit	LCD7      @ (unsigned)&PORTC*8+7 ;

  bit	BSEL      @ (unsigned)&PORTB*8+7 ;
  bit	BMENU     @ (unsigned)&PORTB*8+6 ;
  bit	BA        @ (unsigned)&PORTB*8+5 ; // encoder A
  bit	BB        @ (unsigned)&PORTB*8+0 ; // encoder B

  bit	SLUT;    // to end a sub
  bit   MENU;
  bit   BARGLO;  // if bargraph is hi or lo side
  bit   DISPSPD;
  bit   oldpos;    // encoder A edge detector
  bit   oldmenu;   // menu button, edge detector
  bit   RELATIVE;  // if on, the rf volt and rf power will not be shown
  bit   SWRERROR;  

///////////////////////// GLOBAL CONSTANTS///////////////////////////////

  //                db   0   1   2   3   4   5   6   7
const char swrconv[] = {255,255,255,255,255,255,200,161,132,110,92,78,67,57,49,43,37,32,28,25,22,19,17,15,13,11,10,9,8,7,6,6,5,4,4,3,3,3,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};

const char wten[] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,7,7,7,7,7,7,8,8,8,8,8,9,9,9,9,10};
const char wdec[] = {0,2,5,7,10,12,15,17,20,23,26,29,32,35,38,41,45,48,51,55,58,62,66,70,74,78,82,86,91,95,0,4,9,14,19,24,29,34,40,45,51,57,63,69,75,82,88,95,2,9,16,24,31,39,47,55,63,72,80,89,98,7,17,27,37,47,57,68,79,90,1,13,25,37,50,62,75,89,3,17,31,46,61,76,92,8,24,41,59,76,94,13,32,51,71,91,12,33,55,77,0};

const char vten[] = {22,22,22,23,23,23,23,24,24,24,25,25,25,25,26,26,26,27,27,27,28,28,28,29,29,29,30,30,30,
 31,31,31,32,32,33,33,33,34,34,35,35,35,36,36,37,37,37,38,38,39,39,40,40,41,41,42,42,43,43,44,44,45,
 45,46,46,47,47,48,48,49,50,50,51,51,52,53,53,54,54,55,56,56,57,58,58,59,60,60,61,62,63,63,64,65,65,
 66,67,68,69,69,70,71,72,73,74,74,75,76,77,78,79,80,81,82,83,84,85,85,86,88,89,90,91,92,93,94,95,96,
 97,98,99,101,102,103,104,105,107,108,109,110,112,113,114,116,117,118,120,121,122,124,125,127,128,130,
 131,133,134,136,137,139,141,142,144,146,147,149,151,152,154,156,158,160,161,163,165,167,169,171,173,
 175,177,179,181,183,185,188,190,192,194,197,199,201,203,206,208,211,213,216,218,221,223};

const char vdec[] = {36,62,88,15,41,69,96,24,52,80,9,38,67,97,27,58,88,19,51,83,15,48,81,14,48,82,16,
 51,87,22,59,95,32,70,7,46,84,24,63,3,44,85,26,68,11,54,97,41,86,31,76,22,69,16,64,12,61,10,60,10,62,
 13,65,18,72,26,81,36,92,49,6,64,23,82,42,3,64,26,89,52,17,82,48,14,81,50,18,88,59,30,2,75,49,24,99,
 76,53,31,10,90,71,53,36,20,4,90,77,65,53,43,34,26,19,13,8,4,1,100,99,0,2,5,9,15,21,29,39,49,61,74,88,
 4,21,39,59,80,2,26,52,79,7,37,68,1,35,71,8,47,88,30,74,20,67,16,67,19,74,30,87,47,9,72,37,4,74,45,18,
 93,70,49,30,13,99,86,76,68,62,59,57,58,62,67,75,86,99,14,32,52,75,1,29,60,93,29,68,10,54,2,52,5,61};


#define FALSE	            0
#define TRUE                1
#define OFF	            0
#define ON                  1
#define DISABLE	            0
#define ENABLE	            1
#define SET	            1

// Constants settings for metertype
#define ABOUT               0
#define DC                  2
#define SSB                 3
#define RF                  4
#define RTN                 5

// Contants settings for LCD goto
#define LINE1               0x80
#define LINE2               0xA9
#define CLEAR               0x01
#define GFXSET              0x40

//text matters
// change this to your own callsign and name if you like.
const char TEXT6[]  = "REVE FM LEBANON     "; 
const char TEXT8[]  = "Ver:01.00 2019/11/15";  // version information
const char TEXT9[]  = "Display Update Delay"; 
const char TEXT15[] = "For more information"; 
const char TEXT16[] = "RADIUM 98           "; 
const char TEXT17[] = "Calibrations erased!"; 
const char TEXT18[] = "Not in range +10 -20";  // error massage, if input out of range under calib
const char TEXT19[] = "Select = Activate   ";  // text for menu

//konv.c
void KONVERTER()        
{                               
  user1 = '0';
  user2 = '0';
  user3 = '0';
  user4 = '0';
  while (konv>=1000)          
  {
    konv=konv-1000;
    user1++;
  }
  while (konv>=100)          
  {
    konv=konv-100;
    user2++;
  }
  while (konv>=10)          
  {
    konv=konv-10;
    user3++;
  }
  user4=konv+0x30;   // konvert the rest to ascii
}

//serial.c
// software serial sender is made in assembler to make timing exact
// 4.00 Mhz PIC clock gives 38400 Baud
// software serial sender is needed due to the hardware programming facilities
// in the PIC that uses 2 pins (RX and TX) if the hardware serial module is
// selected, the PCB was wired to the LCD, and only the TX pin was reserved
// long time before the serial out was made...
// By OZ2CPU 2001, 2002 and beyond.

void Ser1(unsigned char user1)
{
  LCDPORT = user1;
  NOP();
  #asm
  bcf  3,5        ; select right ram bank
  bcf  3,6
  goto Serstart
  SerSpeed  // this sub generate the 26uS bit speed for 38400 Baud
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop 
  nop
  nop
  nop
  nop
  nop
  nop
  return
  Serstart
  bcf  7,6        ; RC6=0;  // startbit
  nop
  nop
  call SerSpeed
  btfss  117,0    ; check LC0     1   1
  goto  ser00     ; if zero jump  2   1
  nop             ;                   1
  bsf  7,6        ; RC6 = 1           1*
  goto  ser01     ;                   2
  ser00:
  bcf  7,6        ; RC6 = 0;      1*
  nop             ;               1
  nop             ;               1
  ser01:
  call SerSpeed
  btfss  117,1    ; check LC1
  goto  ser10     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser11
  ser10:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser11:
  call SerSpeed
  btfss  117,2    ; check LC2
  goto  ser20     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser21
  ser20:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser21:
  call SerSpeed
  btfss  117,3    ; check LC3
  goto  ser30     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser31
  ser30:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser31:
  call SerSpeed
  btfss  117,4    ; check LC4
  goto  ser40     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser41
  ser40:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser41:
  call SerSpeed
  btfss  117,5    ; check LC5
  goto  ser50     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser51
  ser50:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser51:
  call SerSpeed
  btfss  117,6    ; check LC6
  goto  ser60     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser61
  ser60:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser61:
  call SerSpeed
  btfss  117,7    ; check LC7
  goto  ser70     ; if zero jump
  nop
  bsf  7,6        ; RC6 = 1
  goto  ser71
  ser70:
  bcf  7,6        ; RC6 = 0;
  nop
  nop
  ser71:
  call SerSpeed
  nop
  nop
  nop
  bsf  7,6        ; RC6 = 1  stopbit
  call SerSpeed
  #endasm
}
//delays.c
// at 4 MHz clock PIC clock freq.
// OZ2CPU Wattmeter

void Del(unsigned char del0)    // input in 10uS           
{
  while (del0>=1)
  {
    CLRWDT();
    del0--;
  }
}

void DelaymS(unsigned char del0)  		//  input in mS
{
  while (del0>=1)
  {
    Del(36);  
    del0--;
  }
}

void Delay(unsigned char del0)  		//  input in mS
{
  while (del0>=1)
  {
    Del(100);  
    del0--;
  }
}

void Delay100(unsigned char del0)  		//  input in mS
{
	while (del0>=1)
	{
	Delay(100);     
	del0--;
	}
}

//LCD.C
void LCDswap()
{
  LCD0 = LC0; 
  LCD1 = LC1; 
  LCD2 = LC2; 
  LCD3 = LC3; 
  LCD4 = LC4; 
  LCD5 = LC5; 
  LCD6 = LC6; 
  LCD7 = LC7; 
}


void LCDskriv1(unsigned char user1)  // write one 1 letter to LCD display
{
   LCDPORT = user1;
   LCDswap();
   ENA = 1;
   ENA = 0;
   Del(1);  // adjusted to max accepted writing speed 
}



void LCDgoto(unsigned char user1)  // moves the invisble cursor position
{
   ENA = 0;
   RS  = 0;
   RW  = 0;
   LCDPORT = user1;
   LCDswap();
   ENA = 1;
   ENA = 0;
   Delay(2);
   RS  = 1;
}



void LCDsetgfx(void)  // user defined graphics carracters, for the bar-graf
{
  LCDgoto(GFXSET);
  
  LCDskriv1(0x00);  // 0 caracter code
  LCDskriv1(0x00);
  LCDskriv1(0x10);
  LCDskriv1(0x10);
  LCDskriv1(0x10);
  LCDskriv1(0x10);
  LCDskriv1(0x10);
  LCDskriv1(0x10);

  LCDskriv1(0x00);  // 1
  LCDskriv1(0x00);
  LCDskriv1(0x18);
  LCDskriv1(0x18);
  LCDskriv1(0x18);
  LCDskriv1(0x18);
  LCDskriv1(0x18);
  LCDskriv1(0x18);

  LCDskriv1(0x00);  // 2
  LCDskriv1(0x00);
  LCDskriv1(0x1c);
  LCDskriv1(0x1c);
  LCDskriv1(0x1c);
  LCDskriv1(0x1c);
  LCDskriv1(0x1c);
  LCDskriv1(0x1c);

  LCDskriv1(0x00);  // 3
  LCDskriv1(0x00);
  LCDskriv1(0x1e);
  LCDskriv1(0x1e);
  LCDskriv1(0x1e);
  LCDskriv1(0x1e);
  LCDskriv1(0x1e);
  LCDskriv1(0x1e);

  LCDskriv1(0x00);  // 4
  LCDskriv1(0x00);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);

  LCDskriv1(0x01);  // 5
  LCDskriv1(0x00);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);
  LCDskriv1(0x1F);

   ENA = 0;
   RS  = 1;
   RW  = 0;
   LCDPORT = 0x80;  // 1.digit
   LCDswap();
   ENA = 1;
   ENA = 0;
   Delay(3);
   RS  = 0;
}


void LCDinit(void)  // init of LCD display
{               // needs to be done on every LCD type, at power up
   ENA = 0;  // LCD control bits is difined in def.h
   RS  = 0;  
   RW  = 0;

   LCDPORT = 0x38;   // RESET, LCDPORT see def.h
   LCDswap();
   Delay(3);         
   ENA = 1; CLRWDT();  
   ENA = 0;
   Delay(20);

   ENA = 1; CLRWDT(); 
   ENA = 0;
   Delay(3);

   ENA = 1;CLRWDT(); 
   ENA = 0;
   Delay(3);

              // Display reset
   ENA = 1;CLRWDT(); 
   ENA = 0;
   Delay(3);


     // Display entry mode set
   LCDPORT = 0x06;  // 4 or 8 bits mode, 8 is choosen
   LCDswap();
   Delay(3);
   ENA = 1;CLRWDT(); 
   ENA = 0;
  
      // Display on
   LCDPORT = 0x0C;  // 0C = no cursor, no blink, defined here.
   LCDswap();
   Delay(3);
   ENA = 1;CLRWDT();
 
   LCDgoto(LINE1);
}



void LCDskriver(unsigned char user2) // write a whole screen 20 x 2 
{                      // input is what text string number
  user1=0;  // pointer for text
  while (user1<20)
   {
     if (user2==6) LCDPORT = TEXT6[user1];
     else
     if (user2==7) LCDPORT = TEXT6[user1];
     else
     if (user2==8) LCDPORT = TEXT8[user1];
     else
     if (user2==9) LCDPORT = TEXT9[user1];
     else
     if (user2==15) LCDPORT = TEXT15[user1];
     else
     if (user2==16) LCDPORT = TEXT16[user1];
     else
     if (user2==17) LCDPORT = TEXT17[user1];
     else
     if (user2==18) LCDPORT = TEXT18[user1];
     else
     if (user2==19) LCDPORT = TEXT19[user1];
     LCDswap();
     ENA = 1;
     ENA = 0;
     Del(1);  
     user1++;
   }
}



void Welcome(void)
{
   LCDgoto(LINE1);
   LCDskriver(6); // the users define lines
   LCDgoto(LINE2);
   LCDskriver(8);
   user1=30;
   while (user1>=1)
   {
     Delay(51); 
     user1--;
   }
  LCDgoto(CLEAR);
}


void KonvSkriv(void)
{
  LCDskriv1(user2);
  LCDskriv1(user3);
  LCDskriv1(user4);
}

//adc.c
// By oz2cpu
// all ADC 

void GetADC(void)
{
    GO_nDONE = 1;        // start RF A level converting
    while(GO_nDONE) CLRWDT();  // wait for AD converter to be done
    rfraw = (unsigned)(ADRESL+(ADRESH*256));   // get result from adres L=8bit H=2bit
    CHS2 = 0; CHS1 = 1;
    CHS0 = 0;  // Select AN2 input RF B 
    if (metertype==SSB)
    {
      if (rfraw > peak)  // if measured value is bigger than displayed value
      {                  // copy value
        peak = rfraw;
        peakholddelay = 50;
      }
      rfraw = peak;  // store the peak value so that all old modules uses this value.
    }
    konv = rfraw;  // RF input A
    KONVERTER();   
    Ser1(user1); Ser1(user2); Ser1(user3); Ser1(user4);
    Ser1(' '); 

    GO_nDONE = 1;        // start RF B level converting
    while(GO_nDONE) CLRWDT();  // wait for AD converter to be done
    rfrawB = (unsigned)(ADRESL+(ADRESH<<8));   // get result from adres L=8bit H=2bit
    CHS2 = 0; CHS1 = 0;
    CHS0 = 0;  // Select AN0 input Volt 
    konv = rfrawB;  // RF input B
    KONVERTER();   
    Ser1(user1); Ser1(user2); Ser1(user3); Ser1(user4);
    Ser1(' '); 

    GO_nDONE = 1;        // start VOLT converting
    while(GO_nDONE) CLRWDT();  // wait for AD converter to be done
    voltraw = (unsigned)(ADRESL+(ADRESH<<8));   // get result from adres L=8bit H=2bit
    CHS2 = 0; CHS1 = 0;
    CHS0 = 1;  // Select AN1 input RF Level 
    konv = voltraw;
    KONVERTER();   
    Ser1(user1); Ser1(user2); Ser1(user3); Ser1(user4);
    Ser1(10);  Ser1(13); 
}
//subs.c
void WriteBand(void)
{
    if (bandmode==0) 
    {
      LCDskriv1(' ');
      LCDskriv1('L');
    }
    else
    if (bandmode==1) 
    {
      LCDskriv1(' ');
      LCDskriv1('H');
    }
    else
    if (bandmode==2) 
    {
      LCDskriv1('V');
      LCDskriv1('H');
    }
    else
    if (bandmode==3) 
    {
      LCDskriv1('U');
      LCDskriv1('H');
    }
    else
    if (bandmode==4) 
    {
      LCDskriv1('S');
      LCDskriv1('H');
    }
    LCDskriv1('F');
}

void MessageNumber(void)
{
    if (messagenumber==0)
    {
      message1 = ' ';
      message2 = 'L';
      message3 = 'F';
      ActualCal = LFcal;
    }
    else
    if (messagenumber==1)
    {
      message1 = ' ';
      message2 = 'H';
      message3 = 'F';
      ActualCal = HFcal;
    }
    else
    if (messagenumber==2)
    {
      message1 = 'V';
      message2 = 'H';
      message3 = 'F';
      ActualCal = VHFcal;
    }
    else
    if (messagenumber==3)
    {
      message1 = 'U';
      message2 = 'H';
      message3 = 'F';
      ActualCal = UHFcal;
    }
    else
    if (messagenumber==4)
    {
      message1 = 'S';
      message2 = 'H';
      message3 = 'F';
      ActualCal = SHFcal;
    }
    else
    if (messagenumber==5)
    {
      message1 = 'I';
      message2 = 'N';
      message3 = 'A';
    }
    else
    if (messagenumber==6)
    {
      message1 = 'I';
      message2 = 'N';
      message3 = 'B';
    }
    else
    if (messagenumber==7)
    {
      message1 = 'A';
      message2 = 'T';
      message3 = 'T';
    }
    else
    if (messagenumber==8)
    {
      message1 = '0';
      message2 = 'd';
      message3 = 'B';
    }
    else
    if (messagenumber==9)
    {
      message1 = '-';
      message2 = '1';
      message3 = '0';
    }
    else
    if (messagenumber==10)
    {
      message1 = '-';
      message2 = '2';
      message3 = '0';
    }
    else
    if (messagenumber==11)
    {
      message1 = '-';
      message2 = '3';
      message3 = '0';
    }
    else
    if (messagenumber==12)
    {
      message1 = '-';
      message2 = '4';
      message3 = '0';
    }
    else
    if (messagenumber==13)
    {
      message1 = '-';
      message2 = '5';
      message3 = '0';
    }
}


void MessageUpdate(void)
{
  messagetimer++;
  if (displaydelay<30) user1 = 50;  // try to keep message update speed kinda constant
  else                              // not dedending on display delay
  user1 = 20;  
  if (messagetimer>=user1)  // info screen speed
  {
    messagetype++;
    messagetimer = 0;
    if (messagetype>=3) messagetype = 0;
  }
  if (messagetype == 0)
  {
    if (bandmode==0) messagenumber = 0;  // LF
    if (bandmode==1) messagenumber = 1;  // HF
    if (bandmode==2) messagenumber = 2;  // VHF
    if (bandmode==3) messagenumber = 3;  // UHF
    if (bandmode==4) messagenumber = 4;  // SHF
  }
  if (messagetype == 1) messagenumber = 7;  // ATT
  if (messagetype == 2)
  {
    if (attmode==0) messagenumber = 8;  // 0dB
    if (attmode==1) messagenumber = 9;  // -10dB
    if (attmode==2) messagenumber = 10;  // -20dB
    if (attmode==3) messagenumber = 11;  // -30dB
    if (attmode==4) messagenumber = 12;  // -40dB
    if (attmode==5) messagenumber = 13;  // -50dB
  }
  MessageNumber();
}


void ZeroCal(void)
{
  LCDgoto(CLEAR);
  LCDgoto(LINE1);
  LCDskriver(17);  // write the Erased info
  eeprom_write(0,0);  // erase first EEPROM location
  for (;;) NOP();   // wait for the watchdog to reset
}

void ReadCal(void)
{
  LCDgoto(CLEAR);
  LCDgoto(LINE1);
  LCDskriv1('C');  LCDskriv1('a');  LCDskriv1('l');  LCDskriv1('i');
  LCDskriv1('b');  LCDskriv1('.');  LCDskriv1('=');  LCDskriv1(' ');
  LCDskriv1('L');
  LCDskriv1('F');
  konv = LFcal;
  KONVERTER();
  KonvSkriv();
  LCDskriv1(' '); 
  LCDskriv1(' ');

  LCDskriv1('H');
  LCDskriv1('F');
  konv = HFcal;
  KONVERTER();
  KonvSkriv();  

  LCDgoto(LINE2);
  LCDskriv1('V');
  LCDskriv1('H');
  LCDskriv1('F');
  konv = VHFcal;
  KONVERTER();
  KonvSkriv(); 
  LCDskriv1(' '); 

  LCDskriv1('U');
  LCDskriv1('H');
  LCDskriv1('F');
  konv = UHFcal;
  KONVERTER();
  KonvSkriv(); 
  LCDskriv1(' '); 

  LCDskriv1('S');
  LCDskriv1('H');
  LCDskriv1('F');
  konv = SHFcal;
  KONVERTER();
  KonvSkriv(); 
  Delay(20);
  while (!BSEL) CLRWDT(); // wait intil button released
  Delay(20);
  while (BSEL) CLRWDT();  // wait for a button to be pressed 
}


void Calibrate(void)
{
  LCDgoto(CLEAR);
  LCDgoto(LINE1);
  if ((rfraw>605)&&(rfraw<850))  // check that input signal is within range.
  {
    LCDskriv1('C'); LCDskriv1('A'); LCDskriv1('L');
    LCDskriv1(' '); LCDskriv1('0'); LCDskriv1('d'); 
    LCDskriv1('B'); LCDskriv1('m'); LCDskriv1('=');
    konv = rfraw;
    KONVERTER();
    KonvSkriv();
    LCDgoto(LINE2);
    LCDskriv1('S'); LCDskriv1('t'); LCDskriv1('o'); LCDskriv1('r');
    LCDskriv1('e'); LCDskriv1('d'); LCDskriv1(' '); LCDskriv1('i');
    LCDskriv1('n'); LCDskriv1(' '); 
    if (bandmode==0) 
    {
      LFcal = rfraw;  // LF
      user1 = rfraw - 600;  // 600 is just the offset for the byte wide EEPROM 
      eeprom_write(0,user1);  // 600 is added again when reading the byte cal data
      LCDskriv1('L'); 
    }
    else
    if (bandmode==1) 
    {
      HFcal = rfraw;  // HF
      user1 = rfraw - 600;  // 600 is just the offset for the byte wide EEPROM 
      eeprom_write(1,user1);  // 600 is added again when reading the byte cal data
      LCDskriv1('H'); 
    }
    else
    if (bandmode==2) 
    {
      VHFcal = rfraw;  // VHF
      user1 = rfraw - 600;  // 600 is just the offset for the byte wide EEPROM 
      eeprom_write(2,user1);  // 600 is added again when reading the byte cal data
      LCDskriv1('V'); LCDskriv1('H');
    }
    else
    if (bandmode==3) 
    {
      UHFcal = rfraw;  // UHF
      user1 = rfraw - 600;  // 600 is just the offset for the byte wide EEPROM 
      eeprom_write(3,user1);  // 700 is added again when reading the byte cal data
      LCDskriv1('U'); LCDskriv1('H'); 
    }
    else
    if (bandmode==4) 
    {
      SHFcal = rfraw;  // SHF
      user1 = rfraw - 600;  // 600 is just the offset for the byte wide EEPROM 
      eeprom_write(4,user1);  // 600 is added again when reading the byte cal data
      LCDskriv1('S'); LCDskriv1('H'); 
    }
    LCDskriv1('F'); LCDskriv1(' ');
    LCDskriv1('B'); LCDskriv1('a'); LCDskriv1('n'); LCDskriv1('d');
  }
  else LCDskriver(18); // write Error message
  Delay100(20);
  messagetimer = 0;
  messagetype = 0;
}


void go_up(void)
{
  if (MENU) // in menu
  {
    menutype++;
    if (menutype>14) menutype = 14; 
  }
  else  // not in menu
  {
    bandmode++;
    if (bandmode>=5) bandmode = 4; 
    messagetimer = 0;
    messagetype = 0;
  }
  if (DISPSPD) displaydelay++;
}


void go_down(void)
{
  if (MENU) // in menu
  {
    menutype--;
    if (menutype>=128) menutype = 0; 
  }
  else
  {
    bandmode--;
    if (bandmode>=128) bandmode = 0; 
    messagetimer = 0;
    messagetype = 0;
  }
  if (DISPSPD) displaydelay--;
}

void EncoderActions(void)  // and buttons
{
    if (!BB) oldpos=0;  // if low
    
    if ((BB) && (oldpos==0)) // if hi and low before, positive edge
    {  
      if (BA) go_up();
      else go_down();
      if (displaydelay>=80) displaydelay = 80;
      if (displaydelay<=2) displaydelay = 2;
      Delay(5);    // debounce delay
      oldpos = 1;  // mark that it's hi
    }
    if (!BMENU) oldmenu = 0;
    if ((BMENU) && (oldmenu==0))
    {
      oldmenu = 1;
      MENU = !MENU;
    }
}


void DisplaySpeed(void)
{
  SLUT = FALSE;
  DISPSPD = TRUE;
  LCDgoto(CLEAR);
  LCDskriver(9);
  while(!SLUT)
  {
    if (!BSEL) SLUT = TRUE;
    EncoderActions();
    LCDgoto(LINE2);
    konv = displaydelay;
    KONVERTER();
    LCDskriv1(user3);
    LCDskriv1(user4);
    LCDskriv1('m');
    LCDskriv1('S');   LCDskriv1(' ');   LCDskriv1(' ');   LCDskriv1(' ');
    LCDskriv1(' ');   LCDskriv1(' ');   LCDskriv1(' ');
    LCDskriv1(' ');   LCDskriv1('S');   LCDskriv1('e');   LCDskriv1('l');
    LCDskriv1('e');   LCDskriv1('c');   LCDskriv1('t');
    LCDskriv1('=');   LCDskriv1('O');   LCDskriv1('K');
  }
  DISPSPD = FALSE;
  eeprom_write(10,displaydelay);  // save value in EEPROM for all eternaty
}



void MenuEnd(void)
{
   MENU = 0;  // exit menu
   Delay(20);
   while (!BSEL) CLRWDT(); // wait intil select button released
   Delay(20);
}

void MenuActions(void)
{
  LCDgoto(CLEAR);
  LCDgoto(LINE1);
  LCDskriv1('M'); LCDskriv1('e'); LCDskriv1('n'); LCDskriv1('u');
  LCDskriv1(':'); LCDskriv1(' ');
  LCDgoto(LINE2);
  LCDskriver(19);  // TEXT : Select = Activate
  while (MENU)
  {
    CLRWDT();
    GetADC();
    EncoderActions();
    LCDgoto(0x85);
    if (menutype==0) 
    {
      LCDskriv1(' '); LCDskriv1('0'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('0'); LCDskriv1('d');
      LCDskriv1('B'); LCDskriv1(' '); LCDskriv1(' ');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 0;
         eeprom_write(11,0);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==1) 
    {
      LCDskriv1(' '); LCDskriv1('1'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('-'); LCDskriv1('1');
      LCDskriv1('0'); LCDskriv1('d'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 1;
         eeprom_write(11,1);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==2) 
    {
      LCDskriv1(' '); LCDskriv1('2'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('-'); LCDskriv1('2');
      LCDskriv1('0'); LCDskriv1('d'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 2;
         eeprom_write(11,2);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==3) 
    {
      LCDskriv1(' '); LCDskriv1('3'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('-'); LCDskriv1('3');
      LCDskriv1('0'); LCDskriv1('d'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 3;
         eeprom_write(11,3);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==4) 
    {
      LCDskriv1(' '); LCDskriv1('4'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('-'); LCDskriv1('4');
      LCDskriv1('0'); LCDskriv1('d'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 4;
         eeprom_write(11,4);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==5) 
    {
      LCDskriv1(' '); LCDskriv1('5'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('t'); LCDskriv1('t');
      LCDskriv1('='); LCDskriv1('-'); LCDskriv1('5');
      LCDskriv1('0'); LCDskriv1('d'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = RF;
         attmode = 5;
         eeprom_write(11,5);  // save attenuator setting
         MenuEnd();
      }
    }
    else
    if (menutype==6) 
    {
      LCDskriv1(' '); LCDskriv1('6'); LCDskriv1(' '); 
      LCDskriv1('D'); LCDskriv1('C'); LCDskriv1(' ');
      LCDskriv1('V'); LCDskriv1('o'); LCDskriv1('l');
      LCDskriv1('t'); LCDskriv1('M'); LCDskriv1('e');
      LCDskriv1('t'); LCDskriv1('e'); LCDskriv1('r');

      if (BSEL==0)  // the select button
      {
         metertype = DC;
         MenuEnd();
      }
    }
    else
    if (menutype==7) 
    {
      LCDskriv1(' '); LCDskriv1('7'); LCDskriv1(' '); 
      LCDskriv1('R'); LCDskriv1('F'); LCDskriv1(' ');
      LCDskriv1('P'); LCDskriv1('o'); LCDskriv1('w');
      LCDskriv1('.'); LCDskriv1('M'); LCDskriv1('e');
      LCDskriv1('t'); LCDskriv1('e'); LCDskriv1('r');

      if (BSEL==0)  // the select button
      {
         metertype = RF;
         MenuEnd();
      }
    }
    else
    if (menutype==8) 
    {
      LCDskriv1(' '); LCDskriv1('8'); LCDskriv1(' '); 
      LCDskriv1('S'); LCDskriv1('S'); LCDskriv1('B');
      LCDskriv1(' '); LCDskriv1('P'); LCDskriv1('E');
      LCDskriv1('P'); LCDskriv1(' '); LCDskriv1('W');
      LCDskriv1('a'); LCDskriv1('t'); LCDskriv1('t');

      if (BSEL==0)  // the select button
      {
         metertype = SSB;
         MenuEnd();
      }
    }

    else
    if (menutype==9) 
    {
      LCDskriv1(' '); LCDskriv1('9'); LCDskriv1(' '); 
      LCDskriv1('R'); LCDskriv1('T'); LCDskriv1('N');
      LCDskriv1('.'); LCDskriv1('L'); LCDskriv1('o');
      LCDskriv1('s'); LCDskriv1('s'); LCDskriv1(' ');
      LCDskriv1('S'); LCDskriv1('W'); LCDskriv1('R');

      if (BSEL==0)  // the select button
      {
         metertype = RTN;
         RelativeCal = rfraw;  // store relative zero value
         MenuEnd();
      }
    }
    else
    if (menutype==10) 
    {
      LCDskriv1('1'); LCDskriv1('0'); LCDskriv1(' '); 
      LCDskriv1('C'); LCDskriv1('a'); LCDskriv1('l');
      LCDskriv1(' '); LCDskriv1('0'); LCDskriv1('d');
      LCDskriv1('B'); LCDskriv1('m'); LCDskriv1(' ');
      WriteBand();
      if (BSEL==0)  // the select button
      {
        Calibrate();
        metertype = RF;
        MENU = 0;
      }
    }
    else
    if (menutype==11) 
    {
      LCDskriv1('1'); LCDskriv1('1'); LCDskriv1(' '); 
      LCDskriv1('R'); LCDskriv1('e'); LCDskriv1('a');
      LCDskriv1('d'); LCDskriv1(' '); LCDskriv1('C');
      LCDskriv1('a'); LCDskriv1('l'); LCDskriv1('D');
      LCDskriv1('a'); LCDskriv1('t'); LCDskriv1('a');
      if (BSEL==0)  // the select button
      {
        ReadCal();
        metertype = RF;
        MENU = 0;
      }
    }
    else
    if (menutype==12) 
    {
      LCDskriv1('1'); LCDskriv1('2'); LCDskriv1(' '); 
      LCDskriv1('Z'); LCDskriv1('e'); LCDskriv1('r');
      LCDskriv1('o'); LCDskriv1(' '); LCDskriv1('A');
      LCDskriv1('l'); LCDskriv1('l'); LCDskriv1(' ');
      LCDskriv1('C'); LCDskriv1('a'); LCDskriv1('l');
      if (BSEL==0)  // the select button
      {
        ZeroCal();  // ends with a watchdog reset !
      }
    }
    else
    if (menutype==13) 
    {
      LCDskriv1('1'); LCDskriv1('3'); LCDskriv1(' '); 
      LCDskriv1('D'); LCDskriv1('i'); LCDskriv1('s');
      LCDskriv1('p'); LCDskriv1('l'); LCDskriv1('a');
      LCDskriv1('y'); LCDskriv1('S'); LCDskriv1('p');
      LCDskriv1('e'); LCDskriv1('e'); LCDskriv1('d');
      if (BSEL==0)  // the select button
      {
        MenuEnd();  // be sure that select button is released
        DisplaySpeed();
        metertype = RF;
        MenuEnd();
      }
    }
    else
    if (menutype==14) 
    {
      LCDskriv1('1'); LCDskriv1('4'); LCDskriv1(' '); 
      LCDskriv1('A'); LCDskriv1('b'); LCDskriv1('o');
      LCDskriv1('u'); LCDskriv1('t'); LCDskriv1(' ');
      LCDskriv1('i'); LCDskriv1('n'); LCDskriv1('f');
      LCDskriv1('o'); LCDskriv1(' '); LCDskriv1(' ');
      if (BSEL==0)  // the select button
      {
         metertype = ABOUT;
         MenuEnd();
      }
    }
  }  // end while menu
  LCDgoto(CLEAR);  // at exit menu
}

//info.c
void InfoWait(void)
{
  user3 = 20; // screen wait time
  while ((user3>=1)&&(SLUT))
  {
    if ((!BSEL)||(!BMENU))  // the select button
    {
      SLUT = 0;
    } 
    Delay100(1);
    user3--;
  }
  while ((!BSEL)||(!BMENU)) CLRWDT();  // wait for button to be released
}






void WriteAbout(void)
{
  SLUT = 1;
  while (SLUT)
  {  
    LCDgoto(LINE1);
    LCDskriver(7);
    LCDgoto(LINE2);
    LCDskriver(8);
    InfoWait();

    LCDgoto(LINE1);
    LCDskriver(15);
    LCDgoto(LINE2);
    LCDskriver(16);
    InfoWait();
  }  // end while about
  metertype = RF;  // select RF powermeter mode after the about screen
}

//measure.c
// All measure and display routines

void WriteWattmeter(void)
{
    if (rfraw2<=99)
    {
      wpointer=rfraw2;  // pointer to Watt table 0-100 positions
      wpref='p';       // pico Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=199)
    {
      wpointer=rfraw2-100;  // pointer to Watt table 0-100 positions
      wpref='p';       // pico Watt
      wpoint = 4;      // where to place the point .
    }
    else
    if (rfraw2<=299)
    {
      wpointer=rfraw2-200;  // pointer to Watt table 0-100 positions
      wpref='n';       // pico Watt
      wpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=399)
    {
      wpointer=rfraw2-300;  // pointer to Watt table 0-100 positions
      wpref='n';       // nano Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=499)
    {
      wpointer=rfraw2-400;  // pointer to Watt table 0-100 positions
      wpref='n';       // nano Watt
      wpoint = 4;      // where to place the point .
    }
    else
    if (rfraw2<=599)
    {
      wpointer=rfraw2-500;  // pointer to Watt table 0-100 positions
      wpref='u';       // micro Watt
      wpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=699)
    {
      wpointer=rfraw2-600;  // pointer to Watt table 0-100 positions
      wpref='u';       // micro Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=799)
    {
      wpointer=rfraw2-700;  // pointer to Watt table 0-100 positions
      wpref='u';       // micro Watt
      wpoint = 4;      // where to place the point .
    }
    else
    if (rfraw2<=899)
    {
      wpointer=rfraw2-800;  // pointer to Watt table 0-100 positions
      wpref='m';       // milli Watt
      wpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=999)
    {
      wpointer=rfraw2-900;  // pointer to Watt table 0-100 positions
      wpref='m';       // milli Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=1099)
    {
      wpointer=rfraw2-1000;  // pointer to Watt table 0-100 positions
      wpref='m';       // milli Watt
      wpoint = 4;      // where to place the point .
    }
    else
    if (rfraw2<=1199)
    {
      wpointer=rfraw2-1100;  // pointer to Watt table 0-100 positions
      wpref=' ';       // Watt
      wpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=1299)
    {
      wpointer=rfraw2-1200;  // pointer to Watt table 0-100 positions
      wpref=' ';       //  Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=1399)
    {
      wpointer=rfraw2-1300;  // pointer to Watt table 0-100 positions
      wpref=' ';       // Watt
      wpoint = 4;      // where to place the point .
    }
    else
    if (rfraw2<=1499)
    {
      wpointer=rfraw2-1400;  // pointer to Watt table 0-100 positions
      wpref='k';       // Watt
      wpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=1599)
    {
      wpointer=rfraw2-1500;  // pointer to Watt table 0-100 positions
      wpref='k';       // Watt
      wpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=1699)
    {
      wpointer=rfraw2-1600;  // pointer to Watt table 0-100 positions
      wpref='k';       // Watt
      wpoint = 4;      // where to place the point .
    }
      w1=wten[wpointer];  // get absolutte part from table
      w2=wdec[wpointer];  // get decimal part from table
      konv = 0;
      konv = w1;
      KONVERTER();
      if (wpoint==4) LCDskriv1(' ');
      LCDskriv1(user4);
      if (wpoint==2) LCDskriv1('.');  // if this is the right place, write the point
      konv = w2;
      KONVERTER();
      LCDskriv1(user3);
      if (wpoint==3) LCDskriv1('.');  // if this is the right place, write the point
      LCDskriv1(user4);
      LCDskriv1(wpref);   
      LCDskriv1('W');  
}


void BarGraph(void)
{
    user2 = 1;  // carracter pointer
    user1 = (rfraw/10);  // now max rf level is 100 
    while (user1>=5)  // how meny full black carracters to draw
    {
      LCDskriv1(4);
      user1 = (unsigned)(user1-5);
      user2++;
    }
    if (user1==5) LCDskriv1(4);  // Draw the last carracter
    else
    if (user1==4) LCDskriv1(3);
    else
    if (user1==3) LCDskriv1(2);
    else
    if (user1==2) LCDskriv1(1);
    else
    if (user1==1) LCDskriv1(0);
}


void BarEraseRest(void)
{
    while (user2<=20)  // erase the rest
    {
      LCDskriv1(' ');
      user2++;
    } 
}

void BarGraphHI(void)  // write the hight part
{
    WriteWattmeter();
    LCDskriv1(' ');

    LCDgoto(0xc7);

    user2 = 7;  // carracter pointer
    user1 = (rfraw/10);   // now max rf level is 100 
    user1 = (unsigned)(user1-5*user2);  // find real starting point
    while (user1>=5)  // how meny full black carracters to draw
    {
      LCDskriv1(4);
      user1=(unsigned)(user1-5);
      user2++;
    }
    if (user1==5) LCDskriv1(4);  // Draw the last carracter
    else
    if (user1==4) LCDskriv1(3);
    else
    if (user1==3) LCDskriv1(2);
    else
    if (user1==2) LCDskriv1(1);
    else
    if (user1==1) LCDskriv1(0);
}


void WriteRFVolt(void)
{
    if (rfraw2<=199)
    {
      vpointer=rfraw2;  // pointer to Watt table 0-100 positions
      vpref='u';       // Mirco Volt
      vpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=399)
    {
      vpointer=rfraw2-200;  // pointer to Watt table 0-100 positions
      vpref='m';       // milli Volt
      vpoint = 1;      // where to place the point .
    }
    else
    if (rfraw2<=599)
    {
      vpointer=rfraw2-400;  // pointer to Watt table 0-100 positions
      vpref='m';       // milli Volt
      vpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=799)
    {
      vpointer=rfraw2-600;  // pointer to Watt table 0-100 positions
      vpref='m';       // milli Volt
      vpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=999)
    {
      vpointer=rfraw2-800;  // pointer to Watt table 0-100 positions
      vpref=' ';       // Volt
      vpoint = 1;      // where to place the point .
    }
    else
    if (rfraw2<=1199)
    {
      vpointer=rfraw2-1000;  // pointer to Watt table 0-100 positions
      vpref=' ';       // Volt
      vpoint = 2;      // where to place the point .
    }
    else
    if (rfraw2<=1399)
    {
      vpointer=rfraw2-1200;  // pointer to Watt table 0-100 positions
      vpref=' ';       // Volt
      vpoint = 3;      // where to place the point .
    }
    else
    if (rfraw2<=1599)
    {
      vpointer=rfraw2-1400;  // pointer to Watt table 0-100 positions
      vpref='k';       // Volt
      vpoint = 1;      // where to place the point .
    }
    else
    if (rfraw2<=1799)
    {
      vpointer=rfraw2-1600;  // pointer to Watt table 0-100 positions
      vpref='k';       // Volt
      vpoint = 2;      // where to place the point .
    }


      v1=vten[vpointer];  // get absolutte part from table
      v2=vdec[vpointer];  // get decimal part from table
      konv = v1;
      KONVERTER();
      if (vpoint==0)
      {
        LCDskriv1('0');
        LCDskriv1('.');  // if this is the right place, write the point
      }
      LCDskriv1(user2);
      if (vpoint==1) LCDskriv1('.');  // if this is the right place, write the point
      LCDskriv1(user3);
      if (vpoint==2) LCDskriv1('.');  // if this is the right place, write the point
      LCDskriv1(user4);
      if (vpoint==3) LCDskriv1('.');  // if this is the right place, write the point

      konv = v2;
      KONVERTER();
      LCDskriv1(user3);
      LCDskriv1(vpref);   
      LCDskriv1('V');  
}


void WriteDBM(void)
{
  if (RELATIVE)
  {      
    if (rfraw>=RelativeCal)
    {
      rfdbm = rfraw-RelativeCal;
      dbswr = 0;  // the return loss and swr value does not have a meaning at +dB
      if (rfdbm>6) SWRERROR = TRUE;
      else SWRERROR = FALSE;
      LCDskriv1('+');  // this can also be changed to a space
    }
    else
    {
      rfdbm = RelativeCal-rfraw;
      dbswr = rfdbm/10;
      SWRERROR = FALSE;
      LCDskriv1('-');  
    }
  }
  else  // absolute dBm read out
  {  
    if (attmode==0) rfRawAtt=rfraw;  // add nothing if no att. is mounted
    if (attmode==1) rfRawAtt=rfraw+100;  // add 10dB if 10dB att. is mounted
    if (attmode==2) rfRawAtt=rfraw+200;  // add 20dB if 20dB att. is mounted
    if (attmode==3) rfRawAtt=rfraw+300;  // add 30dB if 30dB att. is mounted
    if (attmode==4) rfRawAtt=rfraw+400;  // add 40dB if 40dB att. is mounted
    if (attmode==5) rfRawAtt=rfraw+500;  // add 50dB if 50dB att. is mounted

    if (rfRawAtt>=ActualCal)
    {
      rfdbm = rfRawAtt-ActualCal;
      rfraw2 = 800 + rfdbm;  // calibrated raw to volt and power
      LCDskriv1('+');  // this can also be changed to a space
    }
    else
    {
      rfdbm = ActualCal-rfRawAtt;
      rfraw2 = 800 - rfdbm;  // used for the look-up tabels
      LCDskriv1('-');  
    }
  } 
  konv = rfdbm;
  KONVERTER();
  LCDskriv1(user2); LCDskriv1(user3); 
  LCDskriv1('.');  LCDskriv1(user4);  LCDskriv1('d');  LCDskriv1('B'); 

  if (RELATIVE) LCDskriv1(' ');
  else LCDskriv1('m');
}


void WriteDCmeter(void)
{
    konv = voltraw*2;
    KONVERTER(); 
    if (user1=='0')  LCDskriv1(' ');
    else
    LCDskriv1(user1);
    LCDskriv1(user2);  LCDskriv1('.');
    LCDskriv1(user3); LCDskriv1(user4);
    LCDskriv1('V');  LCDskriv1(' ');
    LCDskriv1(' ');  LCDskriv1('S'); LCDskriv1('e'); LCDskriv1('l'); 
    LCDskriv1('e');  LCDskriv1('c'); LCDskriv1('t');
    LCDskriv1('=');  LCDskriv1('R'); LCDskriv1('e'); LCDskriv1('s');
    LCDskriv1('e');  LCDskriv1('t');  

    if (BSEL==0)  // the select button
    {
      voltmin=voltraw;
      voltmax=voltraw;
    }

    if (voltmin>voltraw) voltmin=voltraw;
    if (voltmax<voltraw) voltmax=voltraw;
    LCDgoto(LINE2);
    LCDskriv1('M');    LCDskriv1('i');    LCDskriv1('n'); LCDskriv1('=');
    konv = voltmin*2;
    KONVERTER(); 
    if (user1=='0')  LCDskriv1(' ');
    else
    LCDskriv1(user1);
    LCDskriv1(user2);  LCDskriv1('.');
    LCDskriv1(user3); LCDskriv1(user4);
 
    LCDskriv1(' '); LCDskriv1(' ');

    LCDskriv1('M');    LCDskriv1('a');    LCDskriv1('x'); LCDskriv1('=');
    konv = voltmax*2;
    KONVERTER(); 
    if (user1=='0')  LCDskriv1(' ');
    else
    LCDskriv1(user1);
    LCDskriv1(user2);  LCDskriv1('.');
    LCDskriv1(user3); LCDskriv1(user4);

}


void WriteRFmeter(void)
{
    WriteDBM();
    if (!BSEL)
    {
      while (!BSEL) CLRWDT();  // wait for select to be released 
      RELATIVE=!RELATIVE;     // togle relative bit
      if (RELATIVE) RelativeCal=rfraw;  // store relative zero value
      Delay(20);
    }
    EncoderActions();
    LCDskriv1(' '); 
    if (RELATIVE)
    {
      LCDskriv1(' ');
      LCDskriv1('R');
      LCDskriv1('E');
      LCDskriv1('L');
      LCDskriv1('A');
      LCDskriv1('T');
      LCDskriv1('I');
      LCDskriv1('V');
      LCDskriv1('E');
      LCDskriv1(' ');
      LCDskriv1(' ');
      LCDgoto(LINE2);
      BarGraph(); // and no Wattmeter
      BarEraseRest();
    }
    else
    {
      LCDskriv1(message1);
      LCDskriv1(message2);
      LCDskriv1(message3);
      LCDskriv1(' '); 
      EncoderActions();
      WriteRFVolt();
      EncoderActions();
      LCDskriv1(' '); 
      LCDgoto(LINE2);
      if (rfraw>=575) BARGLO = TRUE;
      if (rfraw<=545) BARGLO = FALSE;
      EncoderActions();
      if (BARGLO)
      {
        BarGraphHI();  // and Wattmeter
        BarEraseRest();
      }
      else
      {
        BarGraph();
        while (user2<=14)  // erase the rest
        {
          LCDskriv1(' ');
          user2++;
        } 
        LCDgoto(0xce);
        WriteWattmeter();
        LCDskriv1(' ');
      }
   }
}



void WriteSSBmeter(void)
{
  WriteDBM();
  EncoderActions();
  LCDskriv1(' '); 
  LCDskriv1(message1);
  LCDskriv1(message2);
  LCDskriv1(message3);
  LCDskriv1(' '); 
  WriteWattmeter(); 
 
  LCDgoto(LINE2);
  BarGraph(); // and no Wattmeter
  BarEraseRest();

  if (peakholddelay >= 10) peakholddelay--;
  else
  {
    if (peak >=10) peak--;  // decay, make sure the peak value can not go under 10
  }
}




void WriteRTNmeter(void)
{
   RELATIVE = 1;
   WriteDBM();
   if (!BSEL)
   {
     while (!BSEL) CLRWDT();  // wait for select to be released 
     RelativeCal = rfraw;  // store relative zero value
     Delay(20);
   }

   LCDskriv1(' ');
   LCDskriv1('R');
   LCDskriv1('E');
   LCDskriv1('T');
   LCDskriv1('U');
   LCDskriv1('R');
   LCDskriv1('N');
   LCDskriv1(' ');
   LCDskriv1('L');
   LCDskriv1('O');
   LCDskriv1('S');
   LCDskriv1('S');

   LCDgoto(LINE2);
   if (dbswr<=60);  // check value is within table size, else crash !!
   {
     user1 = swrconv[dbswr];  // get swr value from table
     konv = user1;
     KONVERTER();
     user5 = '.';
     user2++;  // add 1 to the hundred
     user1=' ';
   }
   if (dbswr>=61)
   {
     user1='-'; user2='-';
     user5='.'; 
     user3='-'; user4='-';
   }
   if (dbswr==0) { user1='C'; user2='A'; user5='L'; user3='E'; user4='D'; }
   else
   if (dbswr==1) { user1='1'; user2='7'; user5='.'; user3='3'; user4='9'; }
   else
   if (dbswr==2) { user1=' '; user2='8'; user5='.'; user3='7'; user4='2'; }
   else
   if (dbswr==3) { user1=' '; user2='5'; user5='.'; user3='8'; user4='4'; }
   else
   if (dbswr==4) { user1=' '; user2='4'; user5='.'; user3='4'; user4='2'; }
   else
   if (dbswr==5) { user1=' '; user2='3'; user5='.'; user3='5'; user4='7'; }
   if (SWRERROR) { user1='E'; user2='R'; user5='R'; user3='O'; user4='R'; }

   LCDskriv1(user1); LCDskriv1(user2);
   LCDskriv1(user5); 
   LCDskriv1(user3); LCDskriv1(user4);
   LCDskriv1(' '); 
   LCDskriv1('S');
   LCDskriv1('W');
   LCDskriv1('R');
   LCDskriv1(' ');
   LCDskriv1(' ');
   LCDskriv1(' ');
   LCDskriv1('S');
   LCDskriv1('E');
   LCDskriv1('L');
   LCDskriv1('=');
   LCDskriv1('Z');
   LCDskriv1('E');
   LCDskriv1('R');
   LCDskriv1('O');
}

//init.c
void INIThardware(void)
{
  TRISB = 0b11100001;  // DISP DATA OUTPUT
  TRISC = 0b00000000; 
  PORTC = 0b01000000;  // TX = 1
  nRBPU = 0;  // port B pull up
  // INIT ADC
  ADCON1 = 0b11100010;    // 10 bit convert ADRESH,ADRESL, PORTA=Analog VCC=REF (PORTE=Digital)
  ADCON0 = 0b01000001;    // 8T clock, Analog input RA0, adon
  PCFG3 = 0;
  PCFG2 = 1;
  PCFG1 = 0;
  PCFG0 = 1;  // AN3 is Vref AN0 and AN1 analog rest digital
  CHS2 = 0; CHS1 = 0;
  CHS0 = 1;  // Select AN1 input RF Level 
  CLRWDT();

 // get cal settings from EEPROM !!!
  user1 = eeprom_read(0);  // get first cal
  if ((user1==0)||(user1==255))  // check if data is bad
  {  
    eeprom_write(0,66);  // erase to normal harware cal 666
    Delay(20);
    eeprom_write(1,66);  // erase to normal harware cal 666
    Delay(20);
    eeprom_write(2,68);  // erase to normal harware cal 668
    Delay(20);
    eeprom_write(3,64);  // erase to normal harware cal 664
    Delay(20);
    eeprom_write(4,62);  // erase to normal harware cal 662
    Delay(20);
    eeprom_write(10,50);  // erase to normal harware cal 50mS  displaydelay
    Delay(20);
    eeprom_write(11,0);  // erase to 0 dB attenuator
    Delay(20);
  }
  user1 = eeprom_read(0);  // get value from EEPROM byte size
  LFcal = 600 + (unsigned int)user1;     // Add 600 to the byte from EEPROM
  user1 = eeprom_read(1);  // 600 was subtracted, before saving value in EEPROM
  HFcal = 600 + (unsigned int)user1;
  user1 = eeprom_read(2);
  VHFcal = 600 + (unsigned int)user1;
  user1 = eeprom_read(3);
  UHFcal = 600 + (unsigned int)user1;
  user1 = eeprom_read(4);
  SHFcal = 600 + (unsigned int)user1;
  displaydelay = eeprom_read(10);  // get value from EEPROM
  attmode = eeprom_read(11);
  Ser1(' '); Ser1(' '); Ser1(10);  Ser1(13);
  Ser1('R'); Ser1('E'); Ser1('V'); Ser1('E'); Ser1(' '); Ser1('F');
  Ser1('M'); Ser1(' '); Ser1('L'); Ser1('E'); Ser1('B'); Ser1('A'); Ser1('N');
  Ser1('O'); Ser1('N'); Ser1(10);  Ser1(13);
  konv = LFcal;
  KONVERTER();
  Ser1(user2); Ser1(user3); Ser1(user4); Ser1('-');
  konv = HFcal;
  KONVERTER();
  Ser1(user2); Ser1(user3); Ser1(user4); Ser1('-');
  konv = VHFcal;
  KONVERTER();
  Ser1(user2); Ser1(user3); Ser1(user4); Ser1('-');
  konv = UHFcal;
  KONVERTER();
  Ser1(user2); Ser1(user3); Ser1(user4); Ser1('-');
  konv = SHFcal;
  KONVERTER();
  Ser1(user2); Ser1(user3); Ser1(user4);
  Ser1(10); Ser1(13);
}

void InitVars(void)
{
  message1 = ' ';
  message2 = ' ';
  message3 = ' ';
  ActualCal = 666;
  bandmode = 0;   // LF
  messagenumber = 0;
  messagetimer = 0;
  messagetype = 0;
  oldmenu = 1;  // at powerup no menu
  metertype = RF;
  RELATIVE = 0;
  MENU = 0;
}


void main(void)
{ 
  INIThardware();  // setup PIC and ports
  LCDinit();
  Welcome();
  LCDsetgfx();  // gfx for the bar-graf
  InitVars();  // also Get vars from EEPROM
  DISPSPD = 0;
  for(;;)  // loop evigt
  {
    GetADC();  // gets all 3 AD inputs and store the values in 3 vars.
    EncoderActions();
    LCDgoto(LINE1);
    EncoderActions();

    if (MENU) MenuActions();  // menu is a higher priority and it controlled by a BIT
    else
    if (metertype==SSB) WriteSSBmeter();
    else
    if (metertype==RF) WriteRFmeter();
    else
    if (metertype==DC) WriteDCmeter();
    else
    if (metertype==RTN) WriteRTNmeter();
    else
    if (metertype==ABOUT) WriteAbout();


    user1 = displaydelay;  // screen update delay
    while (user1>=1)
    {  
      EncoderActions();
      DelaymS(1);
      user1--;
    }
    EncoderActions();
    MessageUpdate();
  }  // end loopen
}    // end main 

